% Input array is generated by a VisualDSP++ 3.5 project.
% tk, 10/05/2004

% Load the generated test data into a MATLAB input array
fftinput = load('test_data.dat');

% Plot the input array
plot (fftinput);

% Calculate the fft of input array
% The result of fft provides a complex vector
fftresult = fft(fftinput);

% Plot the output of the fft
plot (fftresult);

% Get the length of the calculated fft output
% Note: fftresult is a complex vector
N = length(fftresult);

% First component of fftresult is simply the sum of the data,
% therefore we remove it.
fftresult(1)=[];

% Calculate the FFT output magnitude
% Note 1: FFT output magnitude samples are:
% Xmag(m) = |X(m)| = SQRT( X_real(m)^2 + X_img(m)^2 )
% Note 2: FFT input value has been real only, therefore
% the range from 0 <= m <= N/2 is sufficient. If it would
% have been complex, it would go up to N-1.
% Note 3: As fftresult is a complex vector,
% abs() returns the magnitude, which is the same as:
% sqrt(real(X).^2 + imag(X).^2)
% Note 4: .^ = array power
magnitude = abs(fftresult(1:N/2));

% At this point we don't have (yet) the true amplitude of the time-domain signal!
plot(magnitude);

% Adjust to correct amplitude of the time domain signal
% For real inputs:      Scale factor is N/2
% For compler inputs:   Scale factor is N
scaled_mag = magnitude / (N/2);

% Input to the FFT has only real values, therefore
% values m > N/2 are redundant.
nyquist = 1/2;

% Generate/calculate array of values for the x-axis 
% For example: N = 1024, we calculate 512 values from
% 0 to 0.5.
freq = (1:N/2)/(N/2)*nyquist;

plot (freq,scaled_mag);
title('Magnitude of the FFT output');
xlabel('FS');

% Load the FFT result calculated by the DSP
fft65L = load('65L_calculated_mem_dump.dat');
magnitude_65L = abs(fft65L(1:N/2));
scaled_mag_65L = magnitude_65L / (N/2);
% Plot the data
% Note:
% : => dotted line
% o => circles
% b => blue
% : => dotted line
% -. => dash-dot line
% r => red
plot (freq,scaled_mag,':ob',freq,scaled_mag_65L,'-.r');
title('MATLAB and calculated FFT memory dump from DSP');

% Calculate the FFT output power spectrum
% Note 1: FFT power spectrum is:
% Xps(m) = |X(m)|^2 = X_real(m)^2 + X_img(m)^2
% Note 2: FFT input value has been real only, therefore
% the range from 0 <= m <= N/2 is sufficient. If it would
% have been complex, it would go up to N-1.
% Note 3: As fftresult is a complex vector,
% abs() returns the magnitude, which is the same as:
% sqrt(real(X).^2 + imag(X).^2)
% Note 4: .^ = array power
power = abs(fftresult(1:N/2)).^2;

plot(freq,power), grid on
title('Power of the FFT output');
xlabel('FS');